/* Checkweigher library by alelondono@genial.io
 */

#include "Checkweigher.h"

/**
 * Constructor.
 */
Checkweigher::Checkweigher()
{
  
}

void Checkweigher::begin()
{
    // initialize hardware
    Serial.println("called begin");
    bool compare_float(float x, float y, float epsilon = 0.01f){
            if(fabs(x - y) < epsilon)
                return true; //they are same
                return false; //they are not same
    }

    void setIndexes(byte _numChars, byte _dataLength, byte _dataIndex, byte _referLength, byte _referIndex, boolean _refer_enable, byte _norm_cont_contkey, float _min_weight, char _key){
        numChars = _numChars;
        dataLength = _dataLength;
        dataIndex = _dataIndex;
        referLength = _referLength;
        referIndex = _referIndex;
        refer_enable = _refer_enable;
        norm_cont_contkey = _norm_cont_contkey;
        min_weight = _min_weight;
        stable_key[0] = _key;
    } 
}

void Checkweigher::process()
{
    // do something useful
    Serial.println("called process");
    doit();
}

void recvWithEndMarker() {
    static byte ndx = 0;
    char endMarker = '\n';
    char rc;
        
    while (Serial1.available() > 0 && newData == false) {
        rc = Serial1.read();

        if (rc != endMarker) {
            receivedChars[ndx] = rc;
            ndx++;
            if (ndx >= numChars) {
                ndx = numChars - 1;
            }
        }
        else {
            receivedChars[ndx] = '\0'; // terminate the string
            ndx = 0;
            newData = true;
        }
    }
}

void recvWithStartMarker() {
    static byte ndx = 0;
    char startMarker = '=';
    char rc;
    static int equal_counter = 0;
        
    while (Serial1.available() > 0 && newData == false) {
        rc = Serial1.read();
        //Serial.printlnf("before: %i", equal_counter);
        if (rc != startMarker) {
            receivedChars[ndx] = rc;
            //Serial.println("receiving data");
            ndx++;
            if (ndx >= numChars) {
                ndx = numChars - 1;
            }
        }
        else if (rc == startMarker) {
            equal_counter = equal_counter + 1;
            //Serial.printlnf("equal_counter: %i", equal_counter);
        }
        if (equal_counter > 1){
            receivedChars[ndx] = '\0'; // terminate the string
            //Serial.println("terminate string");
            ndx = 0;
            equal_counter = 0;
            newData = true;
        }
    }
}

void showNewData() {
    char refer[referLength];  
    char buffer[dataLength];      
    char buffer_inv[dataLength];   
    byte j = sizeof(buffer) - 1;
    float buffer_inv_float;
    float buffer_float;

    if (newData == true) {
        checkweigher_timestamp = Time.now();
        for (byte i = 0; i < dataLength; i++){
        buffer[i] = receivedChars[dataIndex+i];
        //Serial.println(buffer[i]);
    }
    buffer[dataLength] = '\0';
    //Serial.printlnf("%s",buffer);
    //Serial.println(buffer[dataLength]);
    for (byte i = 0; i < referLength; i++){
        refer[i] = receivedChars[referIndex+i];
    }
    refer[referLength] = '\0';

    for(byte i = 0; i< dataLength; i++){
        buffer_inv[i] = '\0';
    }
    buffer_inv[dataLength] = '\0';

    if (enable_inverse_checkweigher_value){
        for (byte i = 0; i < sizeof(buffer); i++){
            buffer_inv[j] = buffer[i];
            j--;
        }
        buffer_inv[dataLength] = '\0';
        buffer_float = strtof(buffer_inv, NULL);
        //Serial.printlnf("buffer_inv_float: %.1f", buffer_inv_float);
        sprintf(buffer, "%.1f", buffer_float);
        if (buffer_float > 1){
            sensed_weight = TRUE;
        }
        else{
            sensed_weight = FALSE;
        }
    }
    else{
        buffer_float = strtof(buffer, NULL);
        if (buffer_float > 1){
            sensed_weight = TRUE;
        }
        else{
            sensed_weight = FALSE;
        }
    }

    switch(norm_cont_contkey){
        case 1: //comfer
            //Serial.println("Mode 1 - looking for stable weight");
            //Serial.printlnf("bufferFloat: %.4f", buffer_float);
            if((buffer_float >= min_weight)){ 
                if(current_streak < 5 && !streak_reached){//streak not reached yet
                    //Serial.println("current_streak < 5");
                    if(compare_float(buffer_float, current_streak_weight)){ //same as last weight
                        //Serial.println("streak++");
                        current_streak++;
                    }
                    else{
                        //Serial.println("reset streak");
                        current_streak = 0;
                        streak_reached = false;
                        flag_weight =false;
                        current_streak_weight = buffer_float;
                        //Serial.println("flag_weight is false");
                    }
                }
                else if(current_streak >= 5 && !streak_reached){//streak reached                              
                    streak_reached = true;
                    current_streak = 0;
                    flag_weight = true;
                    publish(buffer, refer); 
                }
            }
            else{
                //Serial.println("IGNORE READING");
                streak_reached = false;
                flag_weight = false;
                //Serial.println("flag_weight is false - IGNORE READING");
                current_streak = 0;
            }
            break;
        case 2: //perman
            if(buffer_float >= min_weight){
                if(strcmp(refer, stable_key)==0 && !streak_reached){
                    /*Serial.println("stable");
                    Serial.print("buffer: ");
                    Serial.println(buffer);
                    Serial.println(buffer[dataLength]);*/
                    streak_reached = true;
                    flag_weight = true;
                    publish(buffer, refer);
                }
            }
            else{
                streak_reached = false;
                flag_weight = false;
            }      
            break;
        default:
            if(buffer_float >= min_weight){
                flag_weight = true;
                publish(buffer, refer);
            }
        }
        newData = false;
    }
}

void publish(char _buffer[], char _refer[]){//David: publicamos la referencia con su codigo y su hora
    //Serial.println("Going to Publish the found weight");
    //float _buffer_float = strtof(_buffer, NULL);
    //char data[256] = "";
    char datacat[512] = "";
    snprintf(datacat, sizeof(datacat), "\"wei%i\":", bufferIndex);
    strcat(data, datacat);
    strcat(data, _buffer);
    /*snprintf(datacat, sizeof(datacat), "%.3f", _buffer_float);
    strcat(data, datacat);*/
    snprintf(datacat, sizeof(datacat), ", \"hora%i\":%i000", bufferIndex, checkweigher_timestamp);
    strcat(data, datacat);
    snprintf(datacat, sizeof(datacat), ", \"ref%i\":\"", bufferIndex);
    strcat(data, datacat);
    if (refer_enable){
        strcat(data, _refer);
    }
    else{
        strcat(data, "");
    }
    strcat(data, "\"");
    /*Serial.print(bufferIndex);
    Serial.println(data);*/
    bufferIndex++;
    if(bufferIndex >= bufferSize){
        strcat(data, "}");
        if(waitFor(Particle.connected,10000)) Particle.publish("chkmul",data,PRIVATE);
        bufferIndex = 0;
        memset(data, 0, 256);
        strcat(data, "{");
    }
    else{
        strcat(data, ", ");
        }        
        }
/**
* Example private method
*/
void Checkweigher::doit()
{
    Serial.println("called doit");
}
